package uoc.pfc.danielclemente.navegacioControlador;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


/**
 * Servlet del framework desenvolupat, aquesta és la classe principal <br/>
 * @author Daniel Clemente Marcè
 */
public class NavegacioControlador extends HttpServlet {

    
    private Properties fluxe;
    private Properties accions;
    private Properties vistes;
    private Properties configuracio;
    private Properties rols;
    private Usuari user;

    public NavegacioControlador() {
        fluxe = null;
        accions = null;
        vistes = null;
        configuracio = null;
        rols = null;
        user = null;

    }

    @Override
    public void init(ServletConfig config) throws ServletException {
        super.init(config);

        fluxe = new Properties();
        accions = new Properties();
        vistes = new Properties();
        configuracio = new Properties();
        try {
            String fluxeFitxer =
                    (config.getServletContext().getRealPath("/") + config.getInitParameter("fluxe"));
            fluxe.load(new FileInputStream(fluxeFitxer));
            String accionsFitxer =
                    (config.getServletContext().getRealPath("/") + config.getInitParameter("accions"));
            accions.load(new FileInputStream(accionsFitxer));
            String vistesFitxer =
                    (config.getServletContext().getRealPath("/") + config.getInitParameter("vistes"));
            vistes.load(new FileInputStream(vistesFitxer));
            String configuracioFitxer =
                    (config.getServletContext().getRealPath("/") + config.getInitParameter("configuracio"));
            configuracio.load(new FileInputStream(configuracioFitxer));

            /*
             * Un cop tenim un sistema de login extern, hem de treure
             * el simulador de login
             */
            if (configuracio.containsKey(Constants.SIMULADOR_LOGIN) ){
                if (Constants.PROPIETAT_ACTIVADA.equals(configuracio.getProperty( Constants.SIMULADOR_LOGIN))){
                    user = inicialitzarUsuariSimulat(config);
                }
            }

        } catch (FileNotFoundException e) {
            System.out.println("Problema amb el fitxer de properties:" + e);
        } catch (IOException e) {
            System.out.println("Problema amb el fitxer de properties:" + e);
        } catch (NullPointerException e) {
            System.out.println("Problema amb el fitxer de properties:" + e);
        }
       
    }

    /**
     * Creem un usuari simulat i l'inicialitzem
     * @param config recupera el fitxer de configuració
     * @return un usuari
     * @throws FileNotFoundException No existeix el fitxer de configuracio.properties
     * @throws IOException No pot obrir el fitxer de configuracio.properties
     */
    private Usuari inicialitzarUsuariSimulat(ServletConfig config) throws FileNotFoundException, IOException {

        user = new Usuari();
        user.setName(configuracio.getProperty(Constants.USER_SIMULAT));
        rols = new Properties();
        String rolsFitxer =
                (config.getServletContext().getRealPath("/") + config.getInitParameter("rols"));
        rols.load(new FileInputStream(rolsFitxer));
        user.setRol(llistaRolsToArray(rols.getProperty(user.getName())));
        return user;

    }

    /**
     * Mètode central del fluxe, on es controlen totes les redireccions
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    public void service(HttpServletRequest request,
            HttpServletResponse response) throws ServletException {
             String resultatAccio = "";
             String missatgeError ="";

             if (request.getParameter(Constants.PARAMETRE_ACCIO)==null || "".equals(request.getParameter(Constants.PARAMETRE_ACCIO))){
                    if (request.getParameter(Constants.PARAMETRE_VISTA)!=null && !"".equals(request.getParameter(Constants.PARAMETRE_VISTA))){
                            System.out.println("0");
                        redireccionar(request,response);
                    }else{
                         missatgeError ="No s'ha definit quina acció hem d'executar o a quina vista es vol anar. <br/>Suggerència: "
                                + "introdueixi un paràmetre accio o vista a la jsp que l'ha redirigit fins aquí "
                                + "amb l'acció que necessita executar o la vista a la qual dirigir-se.";
                        anarAError(request, response,missatgeError );
                    } 
             }if (!accions.containsKey(request.getParameter(Constants.PARAMETRE_ACCIO))){
                 missatgeError = "No ha definit l'acció. "
                        + "<br/>Suggerència: "
                        + "Introdueixi al fitxer accions.properties una entrada amb el valor "
                        + request.getParameter(Constants.PARAMETRE_ACCIO)+ " i posi quina classe "
                        + "correspon nompaquet.classe.";
                anarAError(request, response,missatgeError );
             }       
        try {
                missatgeError="No troba la classe a executar. "
                        + "<br/>Suggerència: "
                        + "En el fitxer accions.properties hi ha una entrada amb el valor "
                        + request.getParameter(Constants.PARAMETRE_ACCIO)+ " i correspon a la classe "
                        + accions.getProperty(request.getParameter(Constants.PARAMETRE_ACCIO))
                        + " que no existeix, no pot trobar o no pot executar.";
                resultatAccio = executarAccio(request);

        }catch (InstantiationException ex){
             // error no esta definida l'accio
            anarAError(request, response,  missatgeError);
        }catch (IllegalAccessException ex){
             // error no esta definida l'accio
            anarAError(request, response,  missatgeError);
        }catch (ClassNotFoundException ex){
               // error no esta definida l'accio
            anarAError(request, response,  missatgeError);
        }catch (Exception ex) {
               //error execució
            missatgeError = "La classe ha generat un error. "
                        + "<br/>Suggerència: "
                        + "Revisi la classe :"
                        +accions.getProperty(request.getParameter(Constants.PARAMETRE_ACCIO))
                        + " ja que ha retornat l'error: <br/> "+
                        ex.getMessage();
            anarAError(request, response,  missatgeError);
        }
            anarVista(request, response, resultatAccio);
         return;
    }
    /**
     * Mètode que executa l'acció <br/>
     * recupera el paràmetre accio de la request i l'executa
     * @param request recupera el paràmetre acció
     * @return
     * @throws Exception es produida per la classe executada
     * @throws InstantiationException No pot trobar la classe
     * @throws IllegalAccessException No pot trobar la classe
     * @throws ClassNotFoundException No pot trobar la classe
     */
    private String executarAccio(HttpServletRequest request) throws Exception,InstantiationException,IllegalAccessException, ClassNotFoundException {
        String resultatAccio = "";
        InterficieAccions accio = null;
        String classeExecutar = null;
        classeExecutar = accions.getProperty(request.getParameter(Constants.PARAMETRE_ACCIO));
            accio = (InterficieAccions) Class.forName(classeExecutar).newInstance();
            resultatAccio = accio.principal(request);
       
        return resultatAccio;
    }
        private void redireccionar(HttpServletRequest request, HttpServletResponse response) throws ServletException {
        String vista = request.getParameter(Constants.PARAMETRE_VISTA);
        String missatgeError="";
       if(!vistes.containsKey(vista)){
            missatgeError = "No existeix una entrada per aquesta vista. "
                     + "<br/>Suggerència: "
                     + " En el fitxer vistes.properties no hi ha una entrada amb el valor "
                     + vista + " . Cal crear una entrada amb aquest valor.";
            anarAError(request, response,  missatgeError);
        }
        String jsp = "";
        String rol = "";
        String[] v = vistes.getProperty(vista).split(",");
        jsp = v[0];

        if (v.length>1 && (Constants.PROPIETAT_ACTIVADA.equals(configuracio.getProperty("restriccioUsuari")))) {
            int i = 1;
            while (i < v.length) {
                rol = v[i].toString();
                if (user.conteRol(user, rol)) {

                      i = v.length;
                } else {

                 missatgeError = "El teu usuari no té permisos per accedir a aquesta pàgina. Vostè té els rols :"+user.getRol().toString()
                        + "<br/>Suggerència: "
                        + "En el fitxer vistes.properties hi ha una entrada amb el valor "+ vista
                        + ". Aquest valor de l'entrada ha "
                        + "d'estar seguit per , i el nom del rol que necessiti per accedir. En aquest cas "
                        + "la pàgina "+jsp+" no té el teu rol: "+user.getRol().toString();
                anarAErrorRestriccioUser(request, response,  missatgeError);
                }
                i++;
            }
        }
            try {
                request.getRequestDispatcher(jsp).forward(request, response);
            } catch (IOException ex) {
                Logger.getLogger(NavegacioControlador.class.getName()).log(Level.SEVERE, null, ex);
            }
        return;
    }

    /**
     * Redirigeix a la vista que necessitem anar
     * @param request recupera el paràmetre acció per conseguir saber el destí
     * @param response el necessita per redirigir
     * @param resultatAccio a partir del paràmetre conseguit a la request i aquest <br/>
     *                      valor podem averiguar ho hem de redigir
     * @throws ServletException error general del servlet
     */
    private void anarVista(HttpServletRequest request, HttpServletResponse response, String resultatAccio) throws ServletException {
        String accioExecutada = request.getParameter(Constants.PARAMETRE_ACCIO);
        String missatgeError="";
        if (!fluxe.containsKey(accioExecutada + "." + resultatAccio)){
        missatgeError = "No existeix una entrada per aquesta sortida del programa. "
                        + "<br/>Suggerència: "
                        + "En el fitxer fluxe.properties no hi ha una entrada amb el valor "
                        + accioExecutada + "." + resultatAccio+ ". Cal crear una entrada "
                        + "amb aquest valor.";
            anarAError(request, response,  missatgeError);
        }
        String vistaDestinacio = fluxe.getProperty(accioExecutada + "." + resultatAccio);
        if(!vistes.containsKey(vistaDestinacio)){
            missatgeError = "No existeix una entrada per aquesta vista. "
                     + "<br/>Suggerència: "
                     + " En el fitxer vistes.properties no hi ha una entrada amb el valor "
                     + vistaDestinacio + " . Cal crear una entrada amb aquest valor.";
            anarAError(request, response,  missatgeError);
        }
        String jsp = "";
        String rol = "";
        String[] v = vistes.getProperty(vistaDestinacio).split(",");
        jsp = v[0];

        if (v.length>1 && (Constants.PROPIETAT_ACTIVADA.equals(configuracio.getProperty("restriccioUsuari")))) {
            int i = 1;
            while (i < v.length) {
                rol = v[i].toString();
                if (user.conteRol(user, rol)) {
              
                      i = v.length;
                } else {

                 missatgeError = "El teu usuari no té permisos per accedir a aquesta pàgina. Vostè té els rols :"+user.getRol().toString()
                        + "<br/>Suggerència: "
                        + "En el fitxer vistes.properties hi ha una entrada amb el valor "
                        + accioExecutada + "." + resultatAccio+ ". Aquest valor de l'entrada ha "
                        + "d'estar seguit per , i el nom del rol que necessiti per accedir. En aquest cas "
                        + "la pàgina "+jsp+" no té el teu rol: "+user.getRol().toString();
                anarAErrorRestriccioUser(request, response,  missatgeError);
                }
                i++;
            }
        }
            try {
                request.getRequestDispatcher(jsp).forward(request, response);
            } catch (IOException ex) {
                Logger.getLogger(NavegacioControlador.class.getName()).log(Level.SEVERE, null, ex);
            }
        return;
    }
        /**
         * Redirigeix a una pàgina on mostrarà l'error
         * @param request ho necessita per redirigir
         * @param response ho necessita per redirigir
         * @param MSGerror Text de l'error
         * @throws ServletException error general del servlet
         */
        private void anarAError (HttpServletRequest request, HttpServletResponse response, String MSGerror) throws ServletException {
            if (!isDebugModeActiu()){
                MSGerror = "S'ha produït un error";
            }
            String jsp =Constants.PAGINA_ERROR;
             request.getSession().setAttribute("missatge", MSGerror);
        try {
            request.getRequestDispatcher(jsp).forward(request, response);
        } catch (IOException ex1) {
            Logger.getLogger(NavegacioControlador.class.getName()).log(Level.SEVERE, null, ex1);
        }
             return;
        }
        /**
         * Redirigeix a una pàgina on mostrarà que no té permis per accedir
         * @param request ho necessita per redirigir
         * @param response ho necessita per redirigir
         * @param MSGerror Text de l'error
         * @throws ServletException error general del servlet
         */
        private void anarAErrorRestriccioUser (HttpServletRequest request, HttpServletResponse response, String MSGerror) throws ServletException {
            if (!isDebugModeActiu()){
                MSGerror = "El teu usuari no té permisos per accedir a aquesta pàgina. Vostè té els rols :"+user.getRol().toString();
            }
            String jsp =Constants.PAGINA_ERROR;
             request.getSession().setAttribute("missatge", MSGerror);
        try {
            request.getRequestDispatcher(jsp).forward(request, response);
        } catch (IOException ex1) {
            Logger.getLogger(NavegacioControlador.class.getName()).log(Level.SEVERE, null, ex1);
        }
             return;
        }

        /**
         * A partir d'una llista de rols que ha de ser separada per "," <br/>
         * genera un ArrayList amb un rol (un element separat per ",") a cada element
         * @param llista_rolls Cadena de text separat per comes
         * @return ArrayList on els elements son la llista de rols
         */
      private ArrayList llistaRolsToArray(String llista_rolls) {
        ArrayList llistaArray = new ArrayList();
        String v[] = llista_rolls.split(",");
        llistaArray.addAll(Arrays.asList(v));
        return llistaArray;
    }

      /**
       * Ens indica si el mode Debug esta activat
       * @return si esta activat
       */
      private boolean isDebugModeActiu(){
          boolean resultat = false;
        if (configuracio.containsKey(Constants.MODE_DEBUG)){
            resultat = Constants.PROPIETAT_ACTIVADA.equals(configuracio.getProperty(Constants.MODE_DEBUG));
        }
          return resultat;
      }
}
